# EditFlag.rb (or EditFlag.rbs if scrambled.)
#
# An extension that displays an edit warning flag popup window
# when editing a component, if it's definition has multiple instances.
#
# SketchUp Ruby Extension by Dan Rathbun, AUG 2013.
# Licensed to: Ash Scott, Coastal Concepts.
 
require('sketchup.rb')
 
module CoastalConcepts # <--<<< Ash Scott's top level namespace
 
  module EditFlag  # <--<<< this plugin's sub namespace
 
    ### Constants
    #
    unless defined?(OPT_KEY)
      VERSION  = '2.2.0'
      LOAD_TAG = Module.nesting[0].name << ':' << File.basename(__FILE__)
      OPT_KEY  = 'Plugin_' << Module.nesting[0].name.gsub('::','_')
    end

    ### Module Variables (also see 'EditFlag_loader.rb' for other user vars)
    #
    @@spy = nil

    @@flagitem = nil #unless #defined?(@@flagitem)
    @@flag_use = MF_CHECKED

    @@startup   = true #unless defined?(@@startup)
    @@show_flag = true
    @@mousemenu = false

    @@debug = false

    ### Plugin Module Proxy class ------------------------------------ ###
    #
    class << self
   
      def debug()
        @@debug
      end

      def debug=(arg)
        @@debug = arg ? true : false
      end

      def extension()
        @@extension
      end

      def mousemenu()
        @@mousemenu
      end

      def mousemenu=(arg) # internally used
        bool = arg ? true : false
        @@mousemenu = bool
      end

      def remind()
        @@remind
      end

      def remind?()
        @@remind
      end

      def show_flag()
        @@show_flag
      end

      def show_flag?()
        @@show_flag
      end

      def startup()
        @@startup
      end

      def startup?()
        @@startup
      end

      def startup=(arg) # internally used
        @@startup = arg ? true : false
      end

      def toggle()
        @@startup = false if @@startup
        @@show_flag = !@@show_flag
        @@flag_use =( @@show_flag ? MF_CHECKED : MF_UNCHECKED )
        # save the new setting:
        Sketchup.write_default(OPT_KEY,'show_flag',@@show_flag)
      end
 
    end # proxy class
 
 
    ### Application Observer ----------------------------------------- ###
    #
    class AppSpy < Sketchup::AppObserver
 
      @@parent = Module.nesting[1] # ref to outer namespace
   
      def debug()
        @@parent.debug
      end
   
      def onNewModel(model)
        #
        if debug
          puts()
          puts("onNewModel() called\nmodel: #{model.inspect}")
          puts("show_flag = #{@@parent.show_flag}")
        end
        #
        model.add_observer(@@parent::ModSpy.new(model))
        #
      end
 
      def onOpenModel(model)
        #
        if debug
          puts()
          puts("onOpenModel() called\nmodel: #{model.inspect}")
          puts("show_flag = #{@@parent.show_flag}")
        end
        #
        model.add_observer(@@parent::ModSpy.new(model))
        #
      end
 
      def onQuit()
        # remember the last show_flag setting:
        Sketchup.write_default(@@parent::OPT_KEY,'show_flag',@@parent.show_flag)
      end
 
    end # class AppSpy
 
 
    ### Model Observer ----------------------------------------------- ###
    #
    class ModSpy < Sketchup::ModelObserver
    
      WIN =( RUBY_PLATFORM =~ /(darwin)/i ? false : true )
      
      @@parent = Module.nesting[1] # ref to outer namespace
      @@say = @@parent.module_eval('@@say')
      
      ABSPATH = @@parent::ABSPATH
      HTMLPATH = File.join(ABSPATH,"EditFlag.html")

      def debug()
        @@parent.debug
      end
   
      def initialize(model,*args)
        #
        @dialog = nil
        #
        return self
        #
      end
      
      def close_flag()
        #
        if @dialog # not false if refs a UI::WebDialog object
          #
          @dialog.close if @dialog.visible?
          @dialog = nil # next GC cycle should clean it up
          #
        end
        #
      end
 
      def new_flag(warn)
        #
        dialog = UI::WebDialog.new(@@say[:caption],false,@@parent::OPT_KEY,382,77,0,0,true)
        #
        dialog.set_file( HTMLPATH )
        #
        if WIN # (Trident)
          dialog.add_action_callback('get_flag_text') {|dlg,params|
            dlg.execute_script("document.getElementById('flag').innerText='#{warn}';")
          }
        else # Mac (WebKit)
          dialog.add_action_callback('get_flag_text') {|dlg,params|
            dlg.execute_script("document.getElementById('flag').textContent='#{warn}';")
          }
        end
        #
        return dialog
        #
      end
 
      def init_flag(num)
        #
        warn = @@say[:warn_c].%(num)
        @dialog = new_flag(warn)
        WIN ? @dialog.show : @dialog.show_modal
        #
        puts(warn) if debug
        # ALSO show warning flag in statusbar, while mouse is still:
        Sketchup.set_status_text(warn)
        #
      end
 

      def onActivePathChanged(model)
        #
        editpath = model.active_path # array if in an edit context, nil if not
        if debug
          puts()
          puts("onActivePathChanged() called\neditpath: #{editpath.inspect}")
          puts("show_flag = #{@@parent.show_flag}")
        end
        if editpath
          #
          inst = editpath.last
          if inst && inst.is_a?(Sketchup::ComponentInstance)
            #
            num = inst.definition.count_instances
            #
            @@parent.mousemenu= true  # toggle appears on mouse context menu
            #
            if @@parent.show_flag
              #
              init_flag(num) if num > 1
              #
            else # it is OFF
              #
              if @@parent.startup # check if first edit after startup
                #
                if @@parent.remind # show reminder dialog
                  #
                  choice = UI.messagebox( @@say[:remind], MB_YESNOCANCEL )
                  #
                  if choice == IDYES
                    #
                    @@parent.toggle
                    init_flag(num) if num > 1
                    #
                  elsif choice == IDNO
                    #
                    @@parent.startup= false
                    #
                  end  # Otherwise user cancelled or dismissed the dialog, so do
                  #      nothing ( reminders continue when edit context changes. )
                else
                  #
                  @@parent.startup= false
                  #
                end
                #
              end
              #
            end
            #
          end
          #
        else # not in an edit context:
          #
          @@parent.mousemenu= false  # toggle NOT on mouse context menu
          #
          if @dialog
            Sketchup.set_status_text() # clear status bar message
            close_flag() # close the flag popup
          end
          #
        end
        #
      end # onActivePathChanged()
      
      def onDeleteModel(model)
        if debug
          editpath = model.active_path # array if in an edit context, nil if not
          puts()
          puts("onDeleteModel() called\neditpath: #{editpath.inspect}")
          puts("show_flag = #{@@parent.show_flag}")
        end
      end
      
      def onEraseAll(model)
        #
        # This callback gets called on PC before loading a New or Saved model file.
        #
        if debug
          editpath = model.active_path # array if in an edit context, nil if not
          puts()
          puts("onEraseAll() called\neditpath: #{editpath.inspect}")
          puts("show_flag = #{@@parent.show_flag}")
        end
        #
        close_flag() if @dialog
        #
      end
  
    end # class ModSpy
 
 
    ### Run ONCE at first load --------------------------------------- ###
    #
    unless file_loaded?(LOAD_TAG)
 
      ### Load Options from saved defaults:
      #
      flag = Sketchup.read_default(OPT_KEY,'show_flag')
      if flag.nil?
        # This is the FIRST load of THIS extension
        Sketchup.write_default(OPT_KEY,'show_flag',@@show_flag)
      else
        # flag is either true || false, so
        # restore the last session setting:
        if flag != @@show_flag
          @@show_flag = !@@show_flag
          @@flag_use =( @@show_flag ? MF_CHECKED : MF_UNCHECKED )
          # save the new setting:
          Sketchup.write_default(OPT_KEY,'show_flag',@@show_flag)
        end
      end
      #

      ### Menus:
      #
      @@flagitem = UI::Command.new(@@say[:menuitem]) { toggle() }
      @@flagitem.set_validation_proc() { @@flag_use }
      
      UI.menu(@@topmenu).add_item(@@flagitem)
      
      UI.add_context_menu_handler {|mmenu|
        if @@mousemenu
          mmenu.add_separator
          mmenu.add_item(@@flagitem)
        end
      }
 
      ### Init Observer(s):
      #
      # ... for the FIRST template model opened upon startup:
      @tid = UI.start_timer(1.0,true) {
        if (model = Sketchup.active_model).is_a?(Sketchup::Model) &&
        model.valid? &&
        model.tools.active_tool_id != 0
          UI.stop_timer(@tid)
          model.add_observer(ModSpy.new(model))
        end
      }
      # ... for models opened later manually:
      @@spy = AppSpy.new
      Sketchup.add_observer(@@spy)
 
      ### Mark file as loaded in $loaded_files array:
      #
      file_loaded(LOAD_TAG)
 
      ### cleanup temp vars:
      #
      flag = nil
 
    end
 
  end # module EditFlag
 
end # module CoastalConcepts